import time


def test_pay_unschedule(iamport):
    payload_without_customer_uid = {
        # without 'customer_uid'
        'merchant_uid': 'pay_unschedule_%s' % str(time.time()),
    }

    try:
        iamport.pay_unschedule(**payload_without_customer_uid)
    except KeyError as e:
        assert "['customer_uid'] is required" in str(e)

    payload_full = {
        'customer_uid': '00000000',
        'merchant_uid': 'pay_unschedule_%s' % str(time.time()),
    }

    try:
        iamport.pay_unschedule(**payload_full)
    except iamport.ResponseError as e:
        assert e.code == 1
        assert u'취소할 예약결제 기록이 존재하지 않습니다.' in e.message



from pytest import fixture

from iamport import Iamport

DEFAULT_TEST_IMP_KEY = 'imp_apikey'
DEFAULT_TEST_IMP_SECRET = ('ekKoeW8RyKuT0zgaZsUtXXTLQ4AhPFW3ZGseDA6bkA5lamv9O'
                           'qDMnxyeB9wqOsuO9W3Mx9YSJ4dTqJ3f')


def pytest_addoption(parser):
    parser.addoption('--imp-key', default=DEFAULT_TEST_IMP_KEY,
                     help='iamport client key for testing '
                          '[default: %default]')
    parser.addoption('--imp-secret', default=DEFAULT_TEST_IMP_SECRET,
                     help='iamport secret key for testing '
                          '[default: %default]')


@fixture
def iamport(request):
    imp_key = request.config.getoption('--imp-key')
    imp_secret = request.config.getoption('--imp-secret')
    return Iamport(imp_key=imp_key, imp_secret=imp_secret)



import datetime
import time


def test_prepare(iamport):
    amount = 12000
    mid = 'merchant_%d' % time.mktime(datetime.datetime.now().timetuple())

    result = iamport.prepare(merchant_uid=mid, amount=amount)
    assert result['amount'] == amount
    assert result['merchant_uid'] == mid

    result = iamport.prepare_validate(merchant_uid=mid, amount=amount)
    assert result



def test_pay_onetime(iamport):
    # Without 'card_number'
    payload_notEnough = {
        'merchant_uid': 'qwer1234',
        'amount': 5000,
        'expiry': '2019-03',
        'birth': '500203',
        'pwd_2digit': '19'
    }

    try:
        iamport.pay_onetime(**payload_notEnough)
    except KeyError as e:
        assert "['card_number'] is required" in str(e)

    payload_full = {
        'merchant_uid': 'qwer1234',
        'amount': 5000,
        'card_number': '4092-0230-1234-1234',
        'expiry': '2019-03',
        'birth': '500203',
        'pwd_2digit': '19'
    }

    try:
        iamport.pay_onetime(**payload_full)
    except iamport.ResponseError as e:
        assert e.code == -1
        assert u'카드정보 인증에 실패하였습니다.' in e.message



import functools


def required(required):
    def deco(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            if not isinstance(required, list):
                raise TypeError('required have to be list type')
            compare = set(required) - kwargs.keys()
            if len(compare) != 0:
                raise KeyError(f'{list(compare)} is required')
            return func(*args, **kwargs)
        return wrapper
    return deco



import pytest


def test_find(iamport):
    with pytest.raises(KeyError):
        iamport.find()
    with pytest.raises(iamport.HttpError):
        iamport.find(imp_uid='test')
    with pytest.raises(iamport.HttpError):
        iamport.find(merchant_uid='âàáaā')



import pytest


def test_customer_get(iamport):
    customer_uid = '000000'
    with pytest.raises(iamport.ResponseError) as e:
        iamport.customer_get(customer_uid)
        assert u'요청하신 customer_uid(000000)로 등록된 정보를 찾을 수 없습니다.' == e.message




import json
import requests

from .decorator import required

__all__ = ['IAMPORT_API_URL', 'Iamport']

IAMPORT_API_URL = 'https://api.iamport.kr/'


class Iamport:
    def __init__(self, imp_key, imp_secret, imp_url=IAMPORT_API_URL):
        self.imp_key = imp_key
        self.imp_secret = imp_secret
        self.imp_url = imp_url
        requests_session = requests.Session()
        requests_adapters = requests.adapters.HTTPAdapter(max_retries=3)
        requests_session.mount('https://', requests_adapters)
        self.requests_session = requests_session

    class ResponseError(Exception):
        def __init__(self, code=None, message=None):
            self.code = code
            self.message = message

    class HttpError(Exception):
        def __init__(self, code=None, reason=None):
            self.code = code
            self.reason = reason

    @staticmethod
    def get_response(response):
        if response.status_code != requests.codes.ok:
            raise Iamport.HttpError(response.status_code, response.reason)
        result = response.json()
        if result['code'] != 0:
            raise Iamport.ResponseError(
                result.get('code'), result.get('message')
            )
        return result.get('response')

    def _get_token(self):
        url = f'{self.imp_url}users/getToken'
        payload = {'imp_key': self.imp_key,
                   'imp_secret': self.imp_secret}
        response = self.requests_session.post(url, data=payload)
        return self.get_response(response).get('access_token')

    def get_headers(self):
        return {'X-ImpTokenHeader': self._get_token()}

    def _get(self, url, payload=None):
        headers = self.get_headers()
        response = self.requests_session.get(url,
                                             headers=headers, params=payload)
        return self.get_response(response)

    def _post(self, url, payload=None):
        headers = self.get_headers()
        response = self.requests_session.post(url,
                                              headers=headers, data=payload)
        return self.get_response(response)

    def find_by_merchant_uid(self, merchant_uid):
        url = f'{self.imp_url}payments/find/{merchant_uid}'
        return self._get(url)

    def find_by_imp_uid(self, imp_uid):
        url = f'{self.imp_url}payments/{imp_uid}'
        return self._get(url)

    def find(self, **kwargs):
        merchant_uid = kwargs.get('merchant_uid')
        if merchant_uid:
            return self.find_by_merchant_uid(merchant_uid)
        try:
            imp_uid = kwargs['imp_uid']
        except KeyError:
            raise KeyError('merchant_uid or imp_uid is required')
        return self.find_by_imp_uid(imp_uid)

    def _cancel(self, payload):
        url = f'{self.imp_url}payments/cancel'
        return self._post(url, payload)

    @required(['merchant_uid', 'amount',
               'card_number', 'expiry', 'birth', 'pwd_2digit'])
    def pay_onetime(self, **kwargs):
        url = f'{self.imp_url}subscribe/payments/onetime'
        return self._post(url, kwargs)

    @required(['customer_uid', 'merchant_uid', 'amount'])
    def pay_again(self, **kwargs):
        url = f'{self.imp_url}subscribe/payments/again'
        return self._post(url, kwargs)

    @required(['customer_uid', 'card_number', 'expiry', 'birth'])
    def customer_create(self, **kwargs):
        customer_uid = kwargs.get('customer_uid')
        url = f'{self.imp_url}subscribe/customers/{customer_uid}'
        return self._post(url, kwargs)

    def customer_get(self, customer_uid):
        url = f'{self.imp_url}subscribe/customers/{customer_uid}'
        return self._get(url)

    @required(['merchant_uid', 'amount', 'card_number', 'expiry'])
    def pay_foreign(self, **kwargs):
        url = f'{self.imp_url}subscribe/payments/foreign'
        return self._post(url, kwargs)

    @required(['customer_uid'])
    def pay_schedule(self, **kwargs):
        headers = self.get_headers()
        headers['Content-Type'] = 'application/json'
        url = f'{self.imp_url}subscribe/payments/schedule'
        for key in ['merchant_uid', 'schedule_at', 'amount']:
            for schedules in kwargs['schedules']:
                if key not in schedules:
                    raise KeyError('Essential parameter is missing!: %s' % key)

        response = self.requests_session.post(url, headers=headers,
                                              data=json.dumps(kwargs))
        return self.get_response(response)

    @required(['customer_uid'])
    def pay_unschedule(self, **kwargs):
        url = f'{self.imp_url}subscribe/payments/unschedule'
        return self._post(url, kwargs)

    def cancel_by_merchant_uid(self, merchant_uid, reason, **kwargs):
        payload = {'merchant_uid': merchant_uid, 'reason': reason}
        if kwargs:
            payload.update(kwargs)
        return self._cancel(payload)

    def cancel_by_imp_uid(self, imp_uid, reason, **kwargs):
        payload = {'imp_uid': imp_uid, 'reason': reason}
        if kwargs:
            payload.update(kwargs)
        return self._cancel(payload)

    def cancel(self, reason, **kwargs):
        imp_uid = kwargs.pop('imp_uid', None)
        if imp_uid:
            return self.cancel_by_imp_uid(imp_uid, reason, **kwargs)

        merchant_uid = kwargs.pop('merchant_uid', None)
        if merchant_uid is None:
            raise KeyError('merchant_uid or imp_uid is required')
        return self.cancel_by_merchant_uid(merchant_uid, reason, **kwargs)

    def is_paid(self, amount, **kwargs):
        response = kwargs.get('response')
        if not response:
            response = self.find(**kwargs)
        status = response.get('status')
        response_amount = response.get('amount')
        return status == 'paid' and response_amount == amount

    def prepare(self, merchant_uid, amount):
        url = f'{self.imp_url}payments/prepare'
        payload = {'merchant_uid': merchant_uid, 'amount': amount}
        return self._post(url, payload)

    def prepare_validate(self, merchant_uid, amount):
        url = f'{self.imp_url}payments/prepare/{merchant_uid}'
        response = self._get(url)
        response_amount = response.get('amount')
        return response_amount == amount



def test_is_paid_with_response(iamport):
    mocked_response = {
        'status': 'paid',
        'amount': 1000,
    }
    assert True is iamport.is_paid(amount=1000, response=mocked_response, merchant_uid='test')


def test_is_paid_without_response(iamport):
    assert False is iamport.is_paid(amount=1000, merchant_uid='qwer1234')



from setuptools import setup, find_packages
import codecs

install_requires = ['requests']


def readme():
    with codecs.open('README.rst', encoding='utf-8') as f:
        return f.read()


setup(
    name='iamport-rest-client2',
    version='0.0.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    license='MIT License',
    description="REST client for I'mport;(http://www.iamport.kr)",
    long_description=readme(),
    url='https://github.com/james-song/iamport-rest-client-python3',
    author='RakGyu Song',
    author_email='reeoss@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



import pytest


def test_customer_create(iamport):
    # Without 'card_number'
    payload_notEnough = {
        'customer_uid': 'customer_1234',
        'expiry': '2019-03',
        'birth': '500203',
    }

    with pytest.raises(KeyError) as e:
        iamport.customer_create(**payload_notEnough)
        assert "Essential parameter is missing!: card_number" in str(e)

    payload_full = {
        'customer_uid': 'customer_1234',
        'expiry': '2019-03',
        'birth': '500203',
        'card_number': '4092-0230-1234-1234',
    }

    with pytest.raises(iamport.ResponseError) as e:
        iamport.customer_create(**payload_full)
        assert e.code == -1
        assert u'카드정보 인증 및 빌키 발급에 실패하였습니다.' in e.message



import pytest


def test_cancel(iamport):
    with pytest.raises(TypeError):
        iamport.cancel(imp_uid='nothing')
    with pytest.raises(iamport.ResponseError):
        iamport.cancel('reason', imp_uid='nothing')
    try:
        iamport.cancel('reason', imp_uid='nothing')
    except iamport.ResponseError as e:
        assert e.code == 1
        assert e.message == u'취소할 결제건이 존재하지 않습니다.'


def test_partial_cancel(iamport):
    try:
        iamport.cancel('reason', imp_uid='nothing', amount=100)
    except iamport.ResponseError as e:
        assert e.code == 1
        assert e.message == u'취소할 결제건이 존재하지 않습니다.'


def test_cancel_by_merchant_uid(iamport):
    payload = {
        'merchant_uid': 'any-merchant_uid',
        'reason': 'any-reason',
    }

    try:
        iamport.cancel(**payload)
    except iamport.ResponseError as e:
        assert e.code == 1
        assert e.message == u'취소할 결제건이 존재하지 않습니다.'


def test_cancel_without_merchant_uid(iamport):
    payload = {
        'merchant_uid': None,
        'reason': 'any-reason',
    }

    try:
        iamport.cancel(**payload)
    except KeyError as e:
        assert 'merchant_uid or imp_uid is required' in str(e)


def test_cancel_by_merchant_uid_with_kwargs(iamport):
    payload = {
        'merchant_uid': 'any-merchant_uid',
        'reason': 'any-reason',
        'amount': 1234,
    }

    try:
        iamport.cancel(**payload)
    except iamport.ResponseError as e:
        assert e.code == 1
        assert e.message == u'취소할 결제건이 존재하지 않습니다.'


def test_cancel_by_imp_uid(iamport):
    payload = {
        'imp_uid': 'any-imp_uid',
        'reason': 'any-reason',
    }

    try:
        iamport.cancel(**payload)
    except iamport.ResponseError as e:
        assert e.code == 1
        assert e.message == u'취소할 결제건이 존재하지 않습니다.'



def test_pay_again(iamport):
    # Without 'customer_uid'
    payload_notEnough = {
        'merchant_uid': '1234qwer',
        'amount': 5000,
    }

    try:
        iamport.pay_again(**payload_notEnough)
    except KeyError as e:
        assert "['customer_uid'] is required" in str(e)

    payload_full = {
        'customer_uid': '00000000',
        'merchant_uid': '1234qwer',
        'amount': 5000,
    }

    try:
        iamport.pay_again(**payload_full)
    except iamport.ResponseError as e:
        assert e.code == 1
        assert u'등록되지 않은 구매자입니다.' in e.message



import time


def test_pay_schedule(iamport):
    schedule_at = int(time.time() + 1000)

    payload_without_customer_uid = {
        # without 'customer_uid'
        'schedules': [
            {
                'merchant_uid': 'pay_schedule_%s' % str(time.time()),
                'schedule_at': schedule_at,
                'amount': 2001,
                'name': '주문명1',
                'buyer_name': '주문자명',
                'buyer_email': '주문자 Email주소',
                'buyer_tel': '주문자 전화번호',
                'buyer_addr': '주문자 주소',
                'buyer_postcode': '주문자 우편번호'
            },
        ],
    }

    try:
        iamport.pay_schedule(**payload_without_customer_uid)
    except KeyError as e:
        assert "['customer_uid'] is required" in str(e)

    payload_without_merchant_uid = {
        'customer_uid': '00000000',
        'schedules': [
            {
                # without 'merchant_uid'
                'schedule_at': schedule_at,
                'amount': 10000,
                'name': '주문명2',
                'buyer_name': '주문자명',
                'buyer_email': '주문자 Email주소',
                'buyer_tel': '주문자 전화번호',
                'buyer_addr': '주문자 주소',
                'buyer_postcode': '주문자 우편번호'
            },
        ],
    }

    try:
        iamport.pay_schedule(**payload_without_merchant_uid)
    except KeyError as e:
        assert 'Essential parameter is missing!: merchant_uid' in str(e)

    payload_full = {
        'customer_uid': '00000000',
        'schedules': [
            {
                'merchant_uid': 'pay_schedule_%s' % str(time.time()),
                'schedule_at': schedule_at,
                'amount': 5000,
                'name': '주문명',
                'buyer_name': '주문자명',
                'buyer_email': '주문자 Email주소',
                'buyer_tel': '주문자 전화번호',
                'buyer_addr': '주문자 주소',
                'buyer_postcode': '주문자 우편번호'
            },
        ],
    }

    try:
        iamport.pay_schedule(**payload_full)
    except iamport.ResponseError as e:
        assert e.code == 1
        assert u'등록되지 않은 구매자입니다.' in e.message



def test_pay_foreign(iamport):
    payload = {
        'merchant_uid': 'uid',
        'amount': 100,
        'card_number': 'card-number',
    }
    try:
        iamport.pay_foreign(**payload)
    except KeyError as e:
        assert "['expiry'] is required" in str(e)

    payload.update({
        'expiry': '2016-08',
    })

    try:
        iamport.pay_foreign(**payload)
    except iamport.ResponseError as e:
        assert e.code == -1
